@@ -18,10 +18,20 @@
 {-# OPTIONS_GHC -Wno-redundant-constraints #-}
 module Test.MockCat.Verify where
 
-import Control.Concurrent.STM (TVar, readTVarIO)
+
+import Test.MockCat.Internal.Verify
+  ( tryVerifyCallCount
+  , compareCount
+  , countWithArgsMismatchMessage
+  , doVerify
+  , doVerifyOrder
+  , readInvocationList
+  )
 import Test.MockCat.Internal.Types
-import Control.Monad (guard, when, unless)
-import Data.List (elemIndex, intercalate)
+
+
+import Control.Monad ()
+import Data.List (intercalate)
 import Data.Maybe
 import Test.MockCat.Param
 import Prelude hiding (lookup)
@@ -32,7 +42,7 @@ import Test.MockCat.Cons ((:>))
 import Data.Typeable (Typeable, eqT)
 import Test.MockCat.Internal.MockRegistry (lookupVerifierForFn, withAllUnitGuards)
 import Data.Type.Equality ((:~:) (Refl))
-import Data.Dynamic (fromDynamic)
+import Data.Dynamic (fromDynamic, Dynamic)
 import GHC.TypeLits (TypeError, ErrorMessage(..), Symbol)
 
 -- | Class for verifying mock function.
@@ -45,172 +55,19 @@ verify ::
   VerifyMatchType (ResolvableParamsOf m) ->
   IO ()
 verify m matchType = do
-  ResolvedMock mockName recorder <- requireResolved m
+  candidates <- requireResolved m
+  checkCandidates candidates $ \resolvedMock ->
+    doVerifyResolved resolvedMock matchType
+
+doVerifyResolved :: (Eq params, Show params) => ResolvedMock params -> VerifyMatchType params -> IO (Maybe String)
+doVerifyResolved (ResolvedMock mockName recorder) matchType = do
   invocationList <- readInvocationList (invocationRef recorder)
   case doVerify mockName invocationList matchType of
-    Nothing -> pure ()
-    Just (VerifyFailed msg) ->
-      errorWithoutStackTrace msg `seq` pure ()
-
-doVerify :: (Eq a, Show a) => Maybe MockName -> InvocationList a -> VerifyMatchType a -> Maybe VerifyFailed
-doVerify name list (MatchAny a) = do
-  guard $ notElem a list
-  pure $ verifyFailedMessage name list a
-doVerify name list (MatchAll a) = do
-  guard $ Prelude.any (a /=) list
-  pure $ verifyFailedMessage name list a
-
-readInvocationList :: TVar (InvocationRecord params) -> IO (InvocationList params)
-readInvocationList ref = do
-  record <- readTVarIO ref
-  pure $ invocations record
-
--- | Verify that a resolved mock function was called at least once.
---   This is used internally by typeclass mock verification.
-verifyResolvedAny :: ResolvedMock params -> IO ()
-verifyResolvedAny (ResolvedMock mockName recorder) = do
-  invocationList <- readInvocationList (invocationRef recorder)
-  when (null invocationList) $
-    errorWithoutStackTrace $
-      intercalate
-        "\n"
-        [ "Function" <> mockNameLabel mockName <> " was never called"
-        ]
+    Nothing -> pure Nothing
+    Just (VerifyFailed msg) -> pure (Just msg)
 
 
 
-compareCount :: CountVerifyMethod -> Int -> Bool
-compareCount (Equal e) a = a == e
-compareCount (LessThanEqual e) a = a <= e
-compareCount (LessThan e) a = a < e
-compareCount (GreaterThanEqual e) a = a >= e
-compareCount (GreaterThan e) a = a > e
-
-verifyCount ::
-  ( ResolvableMock m
-  , Eq (ResolvableParamsOf m)
-  ) =>
-  m ->
-  ResolvableParamsOf m ->
-  CountVerifyMethod ->
-  IO ()
-verifyCount m v method = do
-  ResolvedMock mockName recorder <- requireResolved m
-  invocationList <- readInvocationList (invocationRef recorder)
-  let callCount = length (filter (v ==) invocationList)
-  if compareCount method callCount
-    then pure ()
-    else
-      errorWithoutStackTrace $
-        countWithArgsMismatchMessage mockName method callCount
-
--- | Generate error message for count mismatch with arguments
-countWithArgsMismatchMessage :: Maybe MockName -> CountVerifyMethod -> Int -> String
-countWithArgsMismatchMessage mockName method callCount =
-  intercalate
-    "\n"
-    [ "function" <> mockNameLabel mockName <> " was not called the expected number of times with the expected arguments.",
-      "  expected: " <> show method,
-      "   but got: " <> show callCount
-    ]
-
-
-
-verifyOrder ::
-  (ResolvableMock m
-  , Eq (ResolvableParamsOf m)
-  , Show (ResolvableParamsOf m)) =>
-  VerifyOrderMethod ->
-  m ->
-  [ResolvableParamsOf m] ->
-  IO ()
-verifyOrder method m matchers = do
-  ResolvedMock mockName recorder <- requireResolved m
-  invocationList <- readInvocationList (invocationRef recorder)
-  case doVerifyOrder method mockName invocationList matchers of
-    Nothing -> pure ()
-    Just (VerifyFailed msg) ->
-      errorWithoutStackTrace msg `seq` pure ()
-
-doVerifyOrder ::
-  (Eq a, Show a) =>
-  VerifyOrderMethod ->
-  Maybe MockName ->
-  InvocationList a ->
-  [a] ->
-  Maybe VerifyFailed
-doVerifyOrder ExactlySequence name calledValues expectedValues
-  | length calledValues /= length expectedValues = do
-      pure $ verifyFailedOrderParamCountMismatch name calledValues expectedValues
-  | otherwise = do
-      let unexpectedOrders = collectUnExpectedOrder calledValues expectedValues
-      guard $ length unexpectedOrders > 0
-      pure $ verifyFailedSequence name unexpectedOrders
-doVerifyOrder PartiallySequence name calledValues expectedValues
-  | length calledValues < length expectedValues = do
-      pure $ verifyFailedOrderParamCountMismatch name calledValues expectedValues
-  | otherwise = do
-      guard $ isOrderNotMatched calledValues expectedValues
-      pure $ verifyFailedPartiallySequence name calledValues expectedValues
-
-verifyFailedPartiallySequence :: Show a => Maybe MockName -> InvocationList a -> [a] -> VerifyFailed
-verifyFailedPartiallySequence name calledValues expectedValues =
-  VerifyFailed $
-    intercalate
-      "\n"
-      [ "function" <> mockNameLabel name <> " was not called with the expected arguments in the expected order.",
-        "  expected order:",
-        intercalate "\n" $ ("    " <>) . show <$> expectedValues,
-        "  but got:",
-        intercalate "\n" $ ("    " <>) . show <$> calledValues
-      ]
-
-isOrderNotMatched :: Eq a => InvocationList a -> [a] -> Bool
-isOrderNotMatched calledValues expectedValues =
-  isNothing $
-    foldl
-      ( \candidates e -> do
-          candidates >>= \c -> do
-            index <- elemIndex e c
-            Just $ drop (index + 1) c
-      )
-      (Just calledValues)
-      expectedValues
-
-verifyFailedOrderParamCountMismatch :: Maybe MockName -> InvocationList a -> [a] -> VerifyFailed
-verifyFailedOrderParamCountMismatch name calledValues expectedValues =
-  VerifyFailed $
-    intercalate
-      "\n"
-      [ "function" <> mockNameLabel name <> " was not called with the expected arguments in the expected order (count mismatch).",
-        "  expected: " <> show (length expectedValues),
-        "   but got: " <> show (length calledValues)
-      ]
-
-verifyFailedSequence :: Show a => Maybe MockName -> [VerifyOrderResult a] -> VerifyFailed
-verifyFailedSequence name fails =
-  VerifyFailed $
-    intercalate
-      "\n"
-      ( ("function" <> mockNameLabel name <> " was not called with the expected arguments in the expected order.") : (verifyOrderFailedMesssage <$> fails)
-      )
-
-
-
-collectUnExpectedOrder :: Eq a => InvocationList a -> [a] -> [VerifyOrderResult a]
-collectUnExpectedOrder calledValues expectedValues =
-  catMaybes $
-    mapWithIndex
-      ( \i expectedValue -> do
-          let calledValue = calledValues !! i
-          guard $ expectedValue /= calledValue
-          pure VerifyOrderResult {index = i, calledValue = calledValue, expectedValue}
-      )
-      expectedValues
-
-mapWithIndex :: (Int -> a -> b) -> [a] -> [b]
-mapWithIndex f xs = [f i x | (i, x) <- zip [0 ..] xs]
-
 -- Legacy shouldApply* helpers removed. Use shouldBeCalled API instead.
 
 type family PrependParam a rest where
@@ -264,65 +121,16 @@ type ResolvableMock m = (Typeable (ResolvableParamsOf m), Typeable (InvocationRe
 -- | Constraint alias for resolvable mock types with specific params.
 type ResolvableMockWithParams m params = (ResolvableParamsOf m ~ params, ResolvableMock m)
 
-resolveForVerification ::
-  forall target params.
-  ( params ~ ResolvableParamsOf target
-  , Typeable params
-  , Typeable (InvocationRecorder params)
-  ) =>
-  target ->
-  IO (Maybe (Maybe MockName, InvocationRecorder params))
-resolveForVerification target = do
-  let fetch = lookupVerifierForFn target
-  result <-
-    case eqT :: Maybe (params :~: ()) of
-      Just Refl -> withAllUnitGuards fetch
-      Nothing -> fetch
-  case result of
-    Nothing -> pure Nothing
-    Just (name, dynVerifier) ->
-      case fromDynamic @(InvocationRecorder params) dynVerifier of
-        Just verifier -> pure $ Just (name, verifier)
-        Nothing -> pure Nothing
 
 
--- | Verify that a function was called the expected number of times
-verifyCallCount ::
-  Maybe MockName ->
-  InvocationRecorder params ->
-  CountVerifyMethod ->
-  IO ()
-verifyCallCount maybeName recorder method = do
-  invocationList <- readInvocationList (invocationRef recorder)
-  let callCount = length invocationList
-  unless (compareCount method callCount) $
-    errorWithoutStackTrace $
-      countMismatchMessage maybeName method callCount
 
--- | Generate error message for count mismatch
-countMismatchMessage :: Maybe MockName -> CountVerifyMethod -> Int -> String
-countMismatchMessage maybeName method callCount =
-  intercalate
-    "\n"
-    [ "function" <> mockNameLabel maybeName <> " was not called the expected number of times.",
-      "  expected: " <> showCountMethod method,
-      "   but got: " <> show callCount
-    ]
-  where
-    showCountMethod (Equal n) = show n
-    showCountMethod (LessThanEqual n) = "<= " <> show n
-    showCountMethod (GreaterThanEqual n) = ">= " <> show n
-    showCountMethod (LessThan n) = "< " <> show n
-    showCountMethod (GreaterThan n) = "> " <> show n
+
 
 verificationFailure :: IO a
 verificationFailure =
   errorWithoutStackTrace verificationFailureMessage
 
-data ResolvedMock params = ResolvedMock {
-  resolvedMockName :: Maybe MockName,
-  resolvedMockRecorder :: InvocationRecorder params
-}
+
 
 requireResolved ::
   forall target params.
@@ -331,11 +139,38 @@ requireResolved ::
   , Typeable (InvocationRecorder params)
   ) =>
   target ->
-  IO (ResolvedMock params)
+  IO [ResolvedMock params]
 requireResolved target = do
-  resolveForVerification target >>= \case
-    Just (name, recorder) -> pure $ ResolvedMock name recorder
-    Nothing -> verificationFailure
+  candidates <- resolveForVerification target
+  case candidates of
+    [] -> verificationFailure
+    _ -> pure $ map (\(name, recorder) -> ResolvedMock name recorder) candidates
+
+resolveForVerification ::
+  forall target params.
+  ( params ~ ResolvableParamsOf target
+  , Typeable params
+  , Typeable (InvocationRecorder params)
+  ) =>
+  target ->
+  IO [(Maybe MockName, InvocationRecorder params)]
+resolveForVerification target = do
+  let fetch = lookupVerifierForFn target
+  result <-
+    case eqT :: Maybe (params :~: ()) of
+      Just Refl -> withAllUnitGuards fetch
+      Nothing -> fetch
+  findCompatible result
+
+  where
+    findCompatible :: [(Maybe MockName, Dynamic)] -> IO [(Maybe MockName, InvocationRecorder params)]
+    findCompatible [] = pure []
+    findCompatible ((name, dynVerifier) : rest) =
+      case fromDynamic @(InvocationRecorder params) dynVerifier of
+        Just verifier -> do
+          matchRest <- findCompatible rest
+          pure $ (name, verifier) : matchRest
+        Nothing -> findCompatible rest
 
 verificationFailureMessage :: String
 verificationFailureMessage =
@@ -437,8 +272,12 @@ inPartialOrder = OrderSpec PartiallySequence
 --   This accepts both raw values and Param chains.
 --
 --   > f `shouldBeCalled` calledWith "a"
-calledWith :: params -> VerificationSpec params
-calledWith = SimpleVerification
+calledWith ::
+  forall params.
+  (ToNormalizedArg params) =>
+  params ->
+  VerificationSpec (NormalizeWithArg params)
+calledWith = SimpleVerification . toNormalizedArg
 
 -- | Create a simple verification without arguments.
 --   It verifies that the function was called at least once, with ANY arguments.
@@ -517,14 +356,27 @@ inPartialOrderWith args = OrderVerification PartiallySequence (map toNormalizedA
 class ShouldBeCalled m spec where
   shouldBeCalled :: HasCallStack => m -> spec -> IO ()
 
+-- | Helper to verify multiple candidates. Passes if at least one candidate succeeds.
+-- If all fail, throws the first error (or combined error).
+checkCandidates :: [ResolvedMock params] -> (ResolvedMock params -> IO (Maybe String)) -> IO ()
+checkCandidates [] _ = verificationFailure
+checkCandidates candidates verifyFn = do
+  results <- mapM verifyFn candidates
+  let failures = catMaybes results
+  -- If number of failures is less than number of candidates, it means at least one succeeded.
+  if length failures < length candidates
+    then pure ()
+    else errorWithoutStackTrace $ intercalate "\n\n" failures
+
 -- | Instance for times spec alone (without arguments)
 instance
   ( ResolvableMockWithParams m params
   , RequireCallable "shouldBeCalled" m
   ) => ShouldBeCalled m TimesSpec where
   shouldBeCalled m (TimesSpec method) = do
-    ResolvedMock mockName verifier <- requireResolved m
-    verifyCallCount mockName verifier method
+    candidates <- requireResolved m
+    checkCandidates candidates $ \(ResolvedMock mockName verifier) ->
+      tryVerifyCallCount mockName verifier method
 
 -- | Instance for VerificationSpec (handles all verification types)
 instance {-# OVERLAPPING #-}
@@ -533,45 +385,58 @@ instance {-# OVERLAPPING #-}
   , Show params
   , RequireCallable "shouldBeCalled" m
   ) => ShouldBeCalled m (VerificationSpec params) where
-  shouldBeCalled m spec = case spec of
-    CountVerification method args ->
-      verifyCount m args method
-    CountAnyVerification count ->
-      do
-        ResolvedMock mockName recorder <- requireResolved m
-        verifyCallCount mockName recorder count
+  shouldBeCalled m spec = do
+    candidates <- requireResolved m
+    checkCandidates candidates $ \resolvedMock ->
+      verifySpec resolvedMock spec
+
+verifySpec :: (Eq params, Show params) => ResolvedMock params -> VerificationSpec params -> IO (Maybe String)
+verifySpec (ResolvedMock mockName recorder) spec = do
+  invocationList <- readInvocationList (invocationRef recorder)
+  case spec of
+    CountVerification method args -> do
+      let callCount = length (filter (args ==) invocationList)
+      if compareCount method callCount
+        then pure Nothing
+        else pure $ Just $ countWithArgsMismatchMessage mockName method callCount
+    
+    CountAnyVerification method ->
+      tryVerifyCallCount mockName recorder method
+      
     OrderVerification method argsList ->
-      verifyOrder method m argsList
+      case doVerifyOrder method mockName invocationList argsList of
+        Nothing -> pure Nothing
+        Just (VerifyFailed msg) -> pure $ Just msg
+        
     SimpleVerification args ->
-      verify m (MatchAny args)
-    AnyVerification ->
-      do
-        ResolvedMock mockName recorder <- requireResolved m
-        invocationList <- readInvocationList (invocationRef recorder)
-        when (null invocationList) $
-          errorWithoutStackTrace $
-            intercalate
-              "\n"
-              [ "Function" <> mockNameLabel mockName <> " was never called"
-              ]
+      case doVerify mockName invocationList (MatchAny args) of
+        Nothing -> pure Nothing
+        Just (VerifyFailed msg) -> pure $ Just msg
+        
+    AnyVerification -> do
+      if null invocationList
+        then pure $ Just $ intercalate "\n" ["Function" <> mockNameLabel mockName <> " was never called"]
+        else pure Nothing
 
 -- | Instance for Param chains (e.g., "a" ~> "b")
 instance {-# OVERLAPPING #-}
   ( ResolvableMockWithParams m (Param a :> rest)
   , Eq (Param a :> rest)
   , Show (Param a :> rest)
+  , RequireCallable "shouldBeCalled" m
   ) => ShouldBeCalled m (Param a :> rest) where
   shouldBeCalled m args =
-    verify m (MatchAny args)
+    shouldBeCalled m (SimpleVerification args)
 
 -- | Instance for single Param (e.g., param "a")
 instance {-# OVERLAPPING #-}
   ( ResolvableMockWithParams m (Param a)
   , Eq (Param a)
   , Show (Param a)
+  , RequireCallable "shouldBeCalled" m
   ) => ShouldBeCalled m (Param a) where
   shouldBeCalled m args =
-    verify m (MatchAny args)
+    shouldBeCalled m (SimpleVerification args)
 
 -- | Instance for raw values (e.g., "a")
 --   This converts raw values to Param at runtime
@@ -581,6 +446,7 @@ instance {-# OVERLAPPABLE #-}
   , Show (Param a)
   , Show a
   , Eq a
+  , RequireCallable "shouldBeCalled" m
   ) => ShouldBeCalled m a where
   shouldBeCalled m arg =
-    verify m (MatchAny (param arg))
\ No newline at end of file
+    shouldBeCalled m (SimpleVerification (param arg))
