@@ -12,7 +12,8 @@ import Test.Hspec
 import Language.Haskell.TH (pprint, Dec(..), Type(..), litE, stringL, listE, tupE)
 import Language.Haskell.TH.Syntax (nameBase)
 import Data.Char (isDigit, isLower)
-import Data.List (foldl', sort, nub)
+import Data.List (sort, nub)
+import qualified Data.List as L
 import Data.Maybe (mapMaybe)
 import qualified Data.Map.Strict as Map
 import qualified Data.Text as T
@@ -54,6 +55,8 @@ stripQualifiers =
       , "Show."
       , "Classes."
       , "Internal."
+      , "Test.MockCat.Mock."
+      , "Test.MockCat.Types."
       ])
   where
     stripPrefixes [] t = t
@@ -82,6 +85,7 @@ squeezePunctuation =
     , ("((:>)", "(")
     , ("(:>) ", "")
     , ("(:>)", "")
+    , ("([a])", "[a]")
     ]
   . T.replace (T.pack "Head (Param") (T.pack "Head :>Param")
   . T.replace (T.pack "Head :> Param") (T.pack "Head :>Param")
@@ -92,9 +96,17 @@ squeezePunctuation =
   . T.replace (T.pack "Param r r ()") (T.pack "Param r) r ()")
   -- Then handle the general case of ") r ()" -> " r ()" but only if not already fixed
   . T.replace (T.pack ") r ()") (T.pack " r ()")
+  . T.replace (T.pack "Typeable m,") T.empty
+  . T.replace (T.pack ", Typeable m") T.empty
+  . T.replace (T.pack "Typeable a,") T.empty
+  . T.replace (T.pack ", Typeable a") T.empty
+  . T.replace (T.pack " )") (T.pack ")")
+  . T.replace (T.pack "Typeable [a]") (T.pack "Typeable a")
+  . T.replace (T.pack "Typeable ([a])") (T.pack "Typeable a")
+  . T.replace (T.pack "Typeable (ResultType m)") (T.pack "Typeable m")
   where
     applyReplacements reps txt =
-      foldl'
+      L.foldl'
         (\acc (from, to) -> T.replace (T.pack from) (T.pack to) acc)
         txt
         reps
@@ -364,7 +376,7 @@ spec = describe "TH generated vs handwritten instances" do
           expectationFailure "TH generated signature not found: _toUserInput"
         Just sig ->
           normalizeSignature sig
-            `shouldSatisfy` not . T.isInfixOf (T.pack "ResolvableParamsOf") . T.pack
+            `shouldNotSatisfy` T.isInfixOf (T.pack "ResolvableParamsOf") . T.pack
 
     it "_produce signature matches handwritten" $
       assertHelperSigMatches typeClassSpecPath generatedAssocSigMap "_produce"
