@@ -10,12 +10,13 @@
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE ConstraintKinds #-}
-{-# OPTIONS_GHC -Wno-orphans #-}
-{-# OPTIONS_GHC -Wno-name-shadowing #-}
 {-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE DuplicateRecordFields #-}
 {-# LANGUAGE PatternSynonyms #-}
 {-# LANGUAGE OverloadedRecordDot #-}
+{-# OPTIONS_GHC -Wno-orphans #-}
+{-# OPTIONS_GHC -Wno-name-shadowing #-}
+{-# OPTIONS_GHC -Wno-missing-export-lists #-}
 {- HLINT ignore "Use newtype instead of data" -}
 
 module Test.MockCat.TypeClassCommonSpec where
@@ -23,6 +24,7 @@ module Test.MockCat.TypeClassCommonSpec where
 import Prelude hiding (readFile, writeFile, any)
 import Test.Hspec
 import Test.MockCat
+
 import Data.Kind (Type)
 import Data.Text (Text, pack, isInfixOf)
 import Control.Exception (ErrorCall(..), displayException)
@@ -130,9 +132,9 @@ type family ArgsOfF (f :: Type) :: Type where
   ArgsOfF r = ()
 
 -- Generic Mock alias for a function type f
-type MockFor f = forall params. (MockBuilder params f (ArgsOfF f)) => params -> MockT IO f
+type MockFor f = forall params. (MockDispatch (IsMockSpec params) params (MockT IO) f) => params -> MockT IO f
 -- Generic Mock alias for an arbitrary base monad m
-type MockForM m f = forall params. (MockBuilder params f (ArgsOfF f)) => params -> MockT m f
+type MockForM m f = forall params. (MockDispatch (IsMockSpec params) params (MockT m) f) => params -> MockT m f
 
 -- Per-spec dependency records to group required builders/mocks
 data BasicDeps = BasicDeps
@@ -147,7 +149,7 @@ data MixedDeps = MixedDeps
   }
 
 data MultipleDeps = MultipleDeps
-  { _ask      :: Environment -> MockT IO Environment
+  { _ask      :: MockFor Environment
   , _readFile :: MockFor (FilePath -> Text)
   , _writeFile:: MockFor (FilePath -> Text -> ())
   , _post     :: MockFor (Text -> ())
@@ -156,7 +158,7 @@ data MultipleDeps = MultipleDeps
 
 
 data ReaderContextDeps = ReaderContextDeps
-  { _ask      :: Environment -> MockT IO Environment
+  { _ask      :: MockFor Environment
   , _readFile :: MockFor (FilePath -> Text)
   , _writeFile:: MockFor (FilePath -> Text -> ())
   }
@@ -200,11 +202,11 @@ data ExplicitMonadicReturnDeps = ExplicitMonadicReturnDeps
   }
 
 data DefaultMethodDeps = DefaultMethodDeps
-  { _defaultAction :: Int -> MockT IO Int
+  { _defaultAction :: MockFor Int
   }
 
 data AssociatedTypeFamiliesDeps = AssociatedTypeFamiliesDeps
-  { _produce :: Int -> MockT IO Int
+  { _produce :: MockFor Int
   }
 
 data ConcurrencyAndUnliftIODeps = ConcurrencyAndUnliftIODeps
@@ -240,7 +242,7 @@ pattern ExplicitReturnDeps :: MockFor (String -> IO Int) -> MockFor (String -> I
 pattern ExplicitReturnDeps a b <- ExplicitMonadicReturnDeps { _getByExplicit = a, _echoExplicit = b }
   where ExplicitReturnDeps a b = ExplicitMonadicReturnDeps { _getByExplicit = a, _echoExplicit = b }
 
-pattern AssocTypeDeps :: (Int -> MockT IO Int) -> AssociatedTypeFamiliesDeps
+pattern AssocTypeDeps :: MockFor Int -> AssociatedTypeFamiliesDeps
 pattern AssocTypeDeps f <- AssociatedTypeFamiliesDeps { _produce = f }
   where AssocTypeDeps f = AssociatedTypeFamiliesDeps { _produce = f }
 
@@ -346,7 +348,7 @@ specBasicStubbingAndVerification (BasicDeps { _readFile, _writeFile }) = do
 
   it "Program skips file write when input content contains 'ngWord'" do
     result <- runMockT do
-      _ <- _readFile ("input.txt" ~> pack "contains ngWord")
+      _ <- _readFile  ("input.txt" ~> pack "contains ngWord")
       _ <- _writeFile ("output.txt" ~> any ~> ())
         `expects` do
           called never
@@ -366,8 +368,8 @@ specMixedMockingStrategies (MixedDeps { _readFile, _writeFile, _post }) = do
 
     result <- runMockT do
       _ <- _readFile $ "input.txt" ~> pack "content"
-      _ <- _writeFile ("output.text" ~> pack "modifiedContent" ~> ())
-      _ <- _post (pack "modifiedContent" ~> ())
+      _ <- _writeFile $ ("output.text" ~> pack "modifiedContent" ~> ())
+      _ <- _post $ (pack "modifiedContent" ~> ())
       operationProgram2 "input.txt" "output.text" modifyContentStub
 
     result `shouldBe` ()
@@ -388,7 +390,7 @@ specMultipleTypeclassConstraints (MultipleDeps { _ask, _readFile, _writeFile, _p
       _ <- _ask env
       _ <- _readFile ("input.txt" ~> pack "content")
       _ <- _writeFile ("output.text" ~> pack "modifiedContent" ~> ())
-      _ <- _post ((pack "modifiedContent" <> pack ("+" <> show env)) ~> ())
+      _ <- _post (pack "modifiedContent" <> pack ("+" <> show env) ~> ())
       apiFileOperationProgram modifyContentStub
 
     result `shouldBe` ()
@@ -405,8 +407,8 @@ specMonadReaderContextMocking (ReaderContextDeps { _ask, _readFile, _writeFile }
   it "Program successfully uses MonadReader to find paths and executes FileOperation" do
     r <- runMockT do
       _ <- _ask (Environment "input.txt" "output.txt")
-      _ <- _readFile ("input.txt" ~> pack "content")
-      _ <- _writeFile ("output.txt" ~> pack "content" ~> ())
+      _ <- _readFile  ("input.txt" ~> pack "content")
+      _ <- _writeFile  ("output.txt" ~> pack "content" ~> ())
       operationProgram3
     r `shouldBe` ()
 
@@ -563,7 +565,7 @@ specDefaultMethodMocking ::
 specDefaultMethodMocking (DefaultMethodDeps { _defaultAction }) = do
   it "Default method is successfully overridden and stubbed value is returned" do
     result <- runMockT do
-      _ <- _defaultAction (99 :: Int)
+      _ <- _defaultAction (Head :> param (99 :: Int))
       defaultAction
     result `shouldBe` 99
 
@@ -576,7 +578,7 @@ specAssociatedTypeFamiliesSupport ::
 specAssociatedTypeFamiliesSupport (AssociatedTypeFamiliesDeps { _produce }) = do
   it "Associated Type family is correctly resolved and mocked stub value is returned" do
     v <- runMockT do
-      _ <- _produce (321 :: Int)
+      _ <- _produce (Head :> param (321 :: Int))
       produce
     v `shouldBe` 321
 
@@ -651,7 +653,7 @@ specBasicVerificationFailureDetection (BasicDeps { _readFile, _writeFile }) = de
         _ <- _readFile ("input.txt" ~> pack "content")
           `expects` do
             called once
-        _ <- _writeFile ("output.txt" ~> pack "content" ~> ())
+        _ <- _writeFile  ("output.txt" ~> pack "content" ~> ())
         -- readFile is never called, only writeFile is called
         do
           writeFile "output.txt" (pack "content")
@@ -659,7 +661,7 @@ specBasicVerificationFailureDetection (BasicDeps { _readFile, _writeFile }) = de
 
     it "Error when write stub expects call but only readFile is executed" do
       (runMockT @IO do
-        _ <- _readFile ("input.txt" ~> pack "content")
+        _ <- _readFile  ("input.txt" ~> pack "content")
         _ <- _writeFile ("output.txt" ~> pack "content" ~> ())
           `expects` do
             called once
@@ -675,7 +677,7 @@ specMonadReaderVerificationFailureDetection ::
 specMonadReaderVerificationFailureDetection (ReaderContextDeps { _ask }) = describe "verification failures (Reader Environment)" do
     it "Error when MonadReader stub _ask is defined but target function ask is never called" do
       (runMockT @IO do
-        _ <- _ask (Environment "input.txt" "output.txt")
+        _ <- _ask (Head :> param (Environment "input.txt" "output.txt"))
           `expects` do
             called once
         -- ask is never called
@@ -800,14 +802,14 @@ specAdvancedTypesVerificationFailureDetection ::
 specAdvancedTypesVerificationFailureDetection (DefaultMethodDeps { _defaultAction }) (AssociatedTypeFamiliesDeps { _produce }) = describe "verification failures (Default/Assoc)" do
     it "Error when default method stub _defaultAction expects call but defaultAction is never executed" do
       (runMockT @IO do
-        _ <- _defaultAction (99 :: Int)
+        _ <- _defaultAction (Head :> param (99 :: Int))
           `expects` do
             called once
         pure ()) `shouldThrow` missingCall "_defaultAction"
 
     it "Error when associated type stub _produce expects call but produce is never executed" do
       (runMockT @IO do
-        _ <- _produce (321 :: Int)
+        _ <- _produce (Head :> param (321 :: Int))
           `expects` do
             called once
         pure ()) `shouldThrow` missingCall "_produce"
