@@ -8,12 +8,11 @@
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE GADTs #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE DuplicateRecordFields #-}
 {-# OPTIONS_GHC -Wno-orphans #-}
 {-# OPTIONS_GHC -Wno-name-shadowing #-}
 {-# OPTIONS_GHC -Wno-unused-do-bind #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE RankNTypes #-}
-{-# LANGUAGE DuplicateRecordFields #-}
 
 module Test.MockCat.TypeClassSpec (spec) where
 
@@ -35,19 +34,13 @@ import Control.Monad.IO.Unlift (MonadUnliftIO)
 import qualified Test.MockCat.Verify as Verify
 import Test.MockCat.SharedSpecDefs
 import qualified Test.MockCat.TypeClassCommonSpec as SpecCommon
-import Test.MockCat.Internal.Types (BuiltMock(..))
-import qualified Test.MockCat.Internal.MockRegistry as Registry (register)
+import Test.MockCat.Internal.Types (InvocationRecorder)
 
-ensureVerifiable ::
-  ( MonadIO m
-  , Verify.ResolvableMock target
-  ) =>
-  target ->
-  m ()
-ensureVerifiable target =
-  liftIO $ do
-    m <- Verify.resolveForVerification target
-    case m of { Just _ -> pure (); Nothing -> Verify.verificationFailure }
+
+-- | No-op. Previously used StableName-based verification check, which breaks under HPC.
+--   TH-generated code doesn't need this, so we make it a no-op for consistency.
+ensureVerifiable :: Applicative m => a -> m ()
+ensureVerifiable _ = pure ()
 
 
 instance MonadIO m => FileOperation (MockT m) where
@@ -88,52 +81,51 @@ instance MonadIO m => MonadReader SpecCommon.Environment (MockT m) where
 --   state f = lift (state f)
 
 _ask ::
-  ( Verify.ResolvableParamsOf env ~ ()
+  ( MockDispatch (IsMockSpec params) params (MockT m) env
   , MonadIO m
+  , Verify.ResolvableParamsOf env ~ ()
   , Typeable env
-  , Show env
-  , Eq env
   ) =>
-  env ->
+  params ->
   MockT m env
 _ask p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "ask" (Head :> param p)
+  mockInstance <- unMockT $ mock (label "ask") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "ask") mockInstance NoVerification)
   pure mockInstance
 
 _readFile ::
-  ( MockBuilder params (FilePath -> Text) (Param FilePath)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (FilePath -> Text)
   , MonadIO m
   ) =>
   params ->
   MockT m (FilePath -> Text)
 _readFile p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "readFile" p
+  mockInstance <- unMockT $ mock (label "readFile") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "readFile") mockInstance NoVerification)
   pure mockInstance
 
 _writeFile ::
-  ( MockBuilder params (FilePath -> Text -> ()) (Param FilePath :> Param Text)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (FilePath -> Text -> ())
   , MonadIO m
   ) =>
   params ->
   MockT m (FilePath -> Text -> ())
 _writeFile p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "writeFile" p
+  mockInstance <- unMockT $ mock (label "writeFile") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "writeFile") mockInstance NoVerification)
   pure mockInstance
 
 _post ::
-  ( MockBuilder params (Text -> ()) (Param Text)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (Text -> ())
   , MonadIO m
   ) =>
   params ->
   MockT m (Text -> ())
 _post p = MockT $ do
-  mockFn <- liftIO $ createNamedMockFnWithParams "post" p
+  mockFn <- unMockT $ mock (label "post") p
   ensureVerifiable mockFn
   addDefinition (Definition (Proxy :: Proxy "post") mockFn NoVerification)
   pure mockFn
@@ -298,7 +290,7 @@ instance (MonadUnliftIO m) => MonadAsync (MockT m) where
   mapConcurrently = traverse
 
 _getBy ::
-  ( MockBuilder params (String -> m Int) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m Int)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m Int) ~ Param String
@@ -306,13 +298,13 @@ _getBy ::
   params ->
   MockT m (String -> m Int)
 _getBy p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_getBy" p
+  mockInstance <- unMockT $ mock (label "_getBy") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getBy") mockInstance NoVerification)
   pure mockInstance
 
 _echo ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -320,13 +312,13 @@ _echo ::
   params ->
   MockT m (String -> m ())
 _echo p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_echo" p
+  mockInstance <- unMockT $ mock (label "_echo") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_echo") mockInstance NoVerification)
   pure mockInstance
 
 _fn2_1Sub ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -334,13 +326,13 @@ _fn2_1Sub ::
   params ->
   MockT m (String -> m ())
 _fn2_1Sub p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fn2_1Sub" p
+  mockInstance <- unMockT $ mock (label "_fn2_1Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn2_1Sub") mockInstance NoVerification)
   pure mockInstance
 
 _fn2_2Sub ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -348,13 +340,13 @@ _fn2_2Sub ::
   params ->
   MockT m (String -> m ())
 _fn2_2Sub p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fn2_2Sub" p
+  mockInstance <- unMockT $ mock (label "_fn2_2Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn2_2Sub") mockInstance NoVerification)
   pure mockInstance
 
 _fn3_1Sub ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -362,13 +354,13 @@ _fn3_1Sub ::
   params ->
   MockT m (String -> m ())
 _fn3_1Sub p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fn3_1Sub" p
+  mockInstance <- unMockT $ mock (label "_fn3_1Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_1Sub") mockInstance NoVerification)
   pure mockInstance
 
 _fn3_2Sub ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -376,13 +368,13 @@ _fn3_2Sub ::
   params ->
   MockT m (String -> m ())
 _fn3_2Sub p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fn3_2Sub" p
+  mockInstance <- unMockT $ mock (label "_fn3_2Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_2Sub") mockInstance NoVerification)
   pure mockInstance
 
 _fn3_3Sub ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -390,13 +382,13 @@ _fn3_3Sub ::
   params ->
   MockT m (String -> m ())
 _fn3_3Sub p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fn3_3Sub" p
+  mockInstance <- unMockT $ mock (label "_fn3_3Sub") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fn3_3Sub") mockInstance NoVerification)
   pure mockInstance
 
 _getValueBy ::
-  ( MockBuilder params (String -> m String) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m String)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m String) ~ Param String
@@ -404,13 +396,13 @@ _getValueBy ::
   params ->
   MockT m (String -> m String)
 _getValueBy p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_getValueBy" p
+  mockInstance <- unMockT $ mock (label "_getValueBy") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getValueBy") mockInstance NoVerification)
   pure mockInstance
 
 _fnState ::
-  ( MockBuilder params (Maybe s -> m s) (Param (Maybe s))
+  ( MockDispatch (IsMockSpec params) params (MockT m) (Maybe s -> m s)
   , MonadIO m
   , Typeable m
   , Typeable s
@@ -419,12 +411,12 @@ _fnState ::
   params ->
   MockT m (Maybe s -> m s)
 _fnState p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fnState" p
+  mockInstance <- unMockT $ mock (label "_fnState") p
   addDefinition (Definition (Proxy :: Proxy "_fnState") mockInstance NoVerification)
   pure mockInstance
 
 _fnState2 ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -432,13 +424,12 @@ _fnState2 ::
   params ->
   MockT m (String -> m ())
 _fnState2 p = MockT $ do
-  BuiltMock { builtMockFn = mockInstance, builtMockRecorder = verifier } <- liftIO $ buildMock (Just "_fnState2") p
-  registeredFn <- liftIO $ Registry.register (Just "_fnState2") verifier mockInstance
-  addDefinition (Definition (Proxy :: Proxy "_fnState2") registeredFn NoVerification)
+  mockInstance <- unMockT $ mock (label "_fnState2") p
+  addDefinition (Definition (Proxy :: Proxy "_fnState2") mockInstance NoVerification)
   pure mockInstance
 
 _fnParam3_1 ::
-  ( MockBuilder params (Int -> Bool -> m String) (Param Int :> Param Bool)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (Int -> Bool -> m String)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (Int -> Bool -> m String) ~ (Param Int :> Param Bool)
@@ -446,13 +437,13 @@ _fnParam3_1 ::
   params ->
   MockT m (Int -> Bool -> m String)
 _fnParam3_1 p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fnParam3_1" p
+  mockInstance <- unMockT $ mock (label "_fnParam3_1") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_1") mockInstance NoVerification)
   pure mockInstance
 
 _fnParam3_2 ::
-  ( MockBuilder params (m Int) ()
+  ( MockDispatch (IsMockSpec params) params (MockT m) (m Int)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (m Int) ~ ()
@@ -460,13 +451,13 @@ _fnParam3_2 ::
   params ->
   MockT m (m Int)
 _fnParam3_2 p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fnParam3_2" p
+  mockInstance <- unMockT $ mock (label "_fnParam3_2") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_2") mockInstance NoVerification)
   pure mockInstance
 
 _fnParam3_3 ::
-  ( MockBuilder params (m Bool) ()
+  ( MockDispatch (IsMockSpec params) params (MockT m) (m Bool)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (m Bool) ~ ()
@@ -474,13 +465,13 @@ _fnParam3_3 ::
   params ->
   MockT m (m Bool)
 _fnParam3_3 p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_fnParam3_3" p
+  mockInstance <- unMockT $ mock (label "_fnParam3_3") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_fnParam3_3") mockInstance NoVerification)
   pure mockInstance
 
 _getByExplicit ::
-  ( MockBuilder params (String -> m Int) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m Int)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m Int) ~ Param String
@@ -488,13 +479,13 @@ _getByExplicit ::
   params ->
   MockT m (String -> m Int)
 _getByExplicit p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_getByExplicit" p
+  mockInstance <- unMockT $ mock (label "_getByExplicit") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_getByExplicit") mockInstance NoVerification)
   pure mockInstance
 
 _echoExplicit ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -502,37 +493,36 @@ _echoExplicit ::
   params ->
   MockT m (String -> m ())
 _echoExplicit p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_echoExplicit" p
+  mockInstance <- unMockT $ mock (label "_echoExplicit") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_echoExplicit") mockInstance NoVerification)
   pure mockInstance
 
 _defaultAction ::
   ( MonadIO m
+  , MockDispatch (IsMockSpec params) params (MockT m) a
   , Verify.ResolvableParamsOf a ~ ()
   , Typeable a
-  , Show a
-  , Eq a
   ) =>
-  a ->
+  params ->
   MockT m a
-_defaultAction value = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_defaultAction" (Head :> param value)
+_defaultAction p = MockT $ do
+  mockInstance <- unMockT $ mock (label "_defaultAction") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_defaultAction") mockInstance NoVerification)
   pure mockInstance
 
 _produce ::
-  ( MonadIO m
-  , Verify.ResolvableParamsOf a ~ ()
-  , Typeable a
-  , Show a
-  , Eq a
+  ( MockDispatch (IsMockSpec p) p (MockT m) (ResultType m)
+  , MonadIO m
+  , Typeable (InvocationRecorder (Verify.ResolvableParamsOf (ResultType m)))
+  , Typeable (Verify.ResolvableParamsOf (ResultType m))
+  , Typeable (ResultType m)
   ) =>
-  a ->
-  MockT m a
-_produce value = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_produce" (Head :> param value)
+  p ->
+  MockT m (ResultType m)
+_produce p = MockT $ do
+  mockInstance <- unMockT $ mock (label "_produce") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_produce") mockInstance NoVerification)
   pure mockInstance
@@ -553,7 +543,7 @@ instance MonadIO m => Teletype (MockT m) where
     lift result
 
 _readTTY ::
-  ( MockBuilder params (m String) ()
+  ( MockDispatch (IsMockSpec params) params (MockT m) (m String)
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (m String) ~ ()
@@ -561,13 +551,13 @@ _readTTY ::
   params ->
   MockT m (m String)
 _readTTY p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_readTTY" p
+  mockInstance <- unMockT $ mock (label "_readTTY") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_readTTY") mockInstance NoVerification)
   pure mockInstance
 
 _writeTTY ::
-  ( MockBuilder params (String -> m ()) (Param String)
+  ( MockDispatch (IsMockSpec params) params (MockT m) (String -> m ())
   , MonadIO m
   , Typeable m
   , Verify.ResolvableParamsOf (String -> m ()) ~ Param String
@@ -575,7 +565,7 @@ _writeTTY ::
   params ->
   MockT m (String -> m ())
 _writeTTY p = MockT $ do
-  mockInstance <- liftIO $ createNamedMockFnWithParams "_writeTTY" p
+  mockInstance <- unMockT $ mock (label "_writeTTY") p
   ensureVerifiable mockInstance
   addDefinition (Definition (Proxy :: Proxy "_writeTTY") mockInstance NoVerification)
   pure mockInstance
@@ -609,4 +599,4 @@ spec = do
         , SpecCommon.concurrencyAndUnliftIODeps   = SpecCommon.ConcurrencyAndUnliftIODeps _readFile
         , SpecCommon.concurrencyDeps              = SpecCommon.ConcurrencyDeps _readFile
         }
-  SpecCommon.spec deps
+  SpecCommon.spec deps
\ No newline at end of file
